/*____________________________________________________________________________
    pgpLanguage.c

    Copyright(C) 1998,1999 Network Associates, Inc.
    All rights reserved.

	PGP 6.5 Command Line 

    $Id: pgpLanguage.c,v 1.2 1999/05/12 21:01:05 sluu Exp $
____________________________________________________________________________*/

#include <stdio.h>
#include "pgpBase.h"

// S.H {
#include <string.h>
#include "globals.h"
#include "fileio.h"
#ifdef WIN32
#include "windows.h"
#endif /* WIN32 */

#define LINEBUF_SIZE 1024
#define CPM_EOF       0x1A
#define DEFAULT_LANG  "ja"
#define BREAK_CHAR    0x0C
#define COMMNET_CHAR  '#'

#define IsKanjiFirst(c) \
	( (c) >= 0x81U && (c) <= 0x9FU || (c) >= 0xE0U && (c) <= 0xFCU )

#define IsKanjiSecond(c) \
	( (c) >= 0x40U && (c) <= 0xFCU && (c) != 0x7FU )

struct localMessage {
	char *key;
	char *message;
	struct localMessage *next;
};
struct localMessage *messages;
struct localMessage *lastMessage;
// } S.H

char *LANG(char *x) 
{ 
	// S.H {
	struct localMessage *lm;
	
	if( messages != NULL)
	{
		lm = messages;
		do
		{
			if( strcmp( x, lm->key ) == 0 )
				return lm->message;
		}
		while( (lm = lm->next) != NULL);
	}
	
	// } S.H
	return x; 
}

char * get_ext_c_ptr(void) 
{ 
	return NULL; 
}

//void init_charset(void) 
void init_charset(struct pgpmainBones *mainbPtr) 	// S.H
{ 
	// S.H {
	//struct pgpargsPBones *argsPrivbPtr = argsbPtr->argsPrivbPtr;
	struct pgpfileBones  *filebPtr = mainbPtr->filebPtr;
	struct pgpenvBones *envbPtr = filebPtr->envbPtr;
	PGPEnv *env = envbPtr->m_env;
	char *langName;
	char langFileName[ MAX_PATH ];
	char langFilePath[ MAX_PATH ];
	FILE *langFilePtr;
	PGPInt32 pri;
	int ch = 0;
	char ch2 = 0;
	int lineBufCount = 0;
	char inBuffer[ LINEBUF_SIZE ];
	int isKey = TRUE;
	
	messages = NULL;
	lastMessage = NULL;
	
	langName = (char *) pgpenvGetCString( env, PGPENV_LANGUAGE, &pri );
	if( langName == NULL )
		langName = DEFAULT_LANG;
	sprintf( langFileName, "PGPC_%s.lng", langName );
	buildFileName( langFilePath, langFileName );
#ifdef PGP_WIN32
	if( ! fileExists( langFilePath ) )
	{
		HKEY hKey = NULL;
		char langFileNameReg[ MAX_PATH ];
		int langFileNameRegSize;
		int langFileNameRegType;
		if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, "SOFTWARE\\Network Associates\\PGP", 0, KEY_READ, &hKey ) == ERROR_SUCCESS)
		{
			if (RegQueryValueEx( hKey, "Installpath", NULL, &langFileNameRegType, 
								 (LPBYTE)langFileNameReg, &langFileNameRegSize) == ERROR_SUCCESS)
			{
				strncpy( langFilePath, langFileNameReg, langFileNameRegSize );
				if( ( langFileNameRegSize + 1 + strlen( langFileName ) ) <= MAX_PATH )
				{
					if( langFilePath[langFileNameRegSize - 2] != '\\' )
						strcat(langFilePath, "\\");
					strcat(langFilePath, langFileName);
				}
			}
			RegCloseKey(hKey);
		}
	}
#endif
//fprintf( stderr, "lang file=[%s] \n", langFilePath );
	if( fileExists( langFilePath ) )
	{
		if( ( langFilePtr = fopen( langFilePath, FOPRTXT ) ) == NULL )
		{
			fprintf( stderr, LANG("Cannot open language file %s\n"), langFilePath );
			return;
		}

		while( ch != EOF )
		{
			while( ( ( ch = getc( langFilePtr ) ) == '\r' || ch == '\n' )
					&& ch != BREAK_CHAR && ch != EOF && ch != CPM_EOF )
				;

			if( ch == EOF || ch == CPM_EOF )
				break;
			
			if( ch == COMMNET_CHAR )
			{
				while( ch != '\r' && ch != '\n' && ch != CPM_EOF && ch != EOF )
					ch = getc( langFilePtr );
				continue;
			}
			
			if( IsKanjiFirst(ch) )
			{
				
				if( lineBufCount + 1 > LINEBUF_SIZE )
				{
					inBuffer[lineBufCount - 1] = '\0';
					fprintf( stderr, LANG("line '%.30s...' too long\n"), inBuffer );
					break;
				}
				else
				{
					inBuffer[ lineBufCount++ ] = ch;
					ch = getc( langFilePtr );
					if( ch == EOF || ch == CPM_EOF )
						break;
					else if ( IsKanjiSecond(ch) )
					{
						inBuffer[ lineBufCount++ ] = ch;
						continue;
					}
				}
				
			}
			
			if ( ch == BREAK_CHAR )
			{
				char *value;
				if( ( value = (char *) malloc( lineBufCount + 1 ) ) == NULL)
				{
					fprintf( stderr, LANG("\n\007Out of memory.\n") );
					return;
				}
				
				strncpy(value, inBuffer, lineBufCount);
				value[lineBufCount] = '\0';
				
				if ( isKey )
				{
					struct localMessage *lm;
					if( ( lm = (struct localMessage *) malloc( sizeof( lm ) ) ) == NULL)
					{
						fprintf( stderr, LANG("\n\007Out of memory.\n") );
						return;
					}
					
					if( lastMessage == NULL )
						messages = lm;
					else
						lastMessage->next = lm;
					
					lastMessage = lm;
					lm->key  = value;
					lm->next = NULL;
//fprintf( stdout, "key[%s]\n", lastMessage->key );
					lineBufCount = 0;
					isKey = FALSE;
				}
				else
				{
					lastMessage->message = value;
//fprintf( stdout, "message[%s]\n", lastMessage->message );
					lineBufCount = 0;
					isKey = TRUE;
				}
			}
			else if( lineBufCount > LINEBUF_SIZE )
			{
				inBuffer[lineBufCount - 1] = '\0';
				fprintf( stderr, LANG("line '%.30s...' too long\n"), inBuffer );
				break;
			}
			else if( ch == '\\' )
			{
				ch = getc( langFilePtr );
				if( ch != EOF && ch != CPM_EOF )
				{
					switch ( ch )
					{
						case 'a':
							inBuffer[ lineBufCount++ ] = '\a';
							break;
						case 'b':
							inBuffer[ lineBufCount++ ] = '\b';
							break;
						case 'f':
							inBuffer[ lineBufCount++ ] = '\f';
							break;
						case 'n':
							inBuffer[ lineBufCount++ ] = '\n';
							break;
						case 'r':
							inBuffer[ lineBufCount++ ] = '\r';
							break;
						case 't':
							inBuffer[ lineBufCount++ ] = '\t';
							break;
						case 'v':
							inBuffer[ lineBufCount++ ] = '\v';
							break;
						case '\\':
							inBuffer[ lineBufCount++ ] = '\\';
							break;
						case '\"':
							inBuffer[ lineBufCount++ ] = '\"';
							break;
						case '\'':
							inBuffer[ lineBufCount++ ] = '\'';
							break;
						case '0':
						case '1':
						case '2':
						case '3':
						case '4':
						case '5':
						case '6':
						case '7':
							{
								char oct[4];
								if( ( ( oct[1] = getc( langFilePtr ) ) != EOF && oct[1] != CPM_EOF ) &&
									( ( oct[2] = getc( langFilePtr ) ) != EOF && oct[2] != CPM_EOF ) )
								{
									int c = 0;
									oct[0] = ch; oct[3] = '\0';
									c = atoi( oct );
									{
										c = (((c - (c % 100)) / 100) * 64) + ((((c - (c % 10)) % 100) / 10) * 8) + (c % 10);
										inBuffer[ lineBufCount++ ] = c;
									}
								}
							}
							break;
						case 'X':
							{
								char hex[2];
								if( ( ( hex[0] = getc( langFilePtr ) ) != EOF && hex[0] != CPM_EOF ) &&
									( ( hex[1] = getc( langFilePtr ) ) != EOF && hex[1] != CPM_EOF ) )
								{
									int c = 0, i;
									int v[2];
									for( i = 0; i < 2; i++ )
									{
										switch ( hex[i] )
										{
											case '0': v[i] = 0; break;
											case '1': v[i] = 1; break;
											case '2': v[i] = 2; break;
											case '3': v[i] = 3; break;
											case '4': v[i] = 4; break;
											case '5': v[i] = 5; break;
											case '6': v[i] = 6; break;
											case '7': v[i] = 7; break;
											case '8': v[i] = 8; break;
											case '9': v[i] = 9; break;
											case 'a': case 'A': v[i] = 10; break;
											case 'b': case 'B': v[i] = 11; break;
											case 'c': case 'C': v[i] = 12; break;
											case 'd': case 'D': v[i] = 13; break;
											case 'e': case 'E': v[i] = 14; break;
											case 'f': case 'F': v[i] = 15; break;
											default: v[i] = 0;
										}
									}
									c = (v[0] * 16) + v[1];
									if( c != 0 )
										inBuffer[ lineBufCount++ ] = c;
								}
							}
							break;
						default:
							inBuffer[ lineBufCount++ ] = ch;
					}
				}
			}
			else
				inBuffer[ lineBufCount++ ] = ch;
		}
		fclose( langFilePtr );
	}
	
	// } S.H
	;
}

int set_CONVERSION() 
{ 
	return 0; 
}

int get_CONVERSION() 
{ 
	return 0; 
}

void CONVERT_TO_CANONICAL_CHARSET(char *s) 
{ 
	;
}

char EXT_C(char c) 
{ 
	return c; 
}

char INT_C(char c) 
{ 
	return c; 
}



