/*____________________________________________________________________________
	Copyright (C) 1999 Seigo Hashiya, Bote
	All rights reserved.

	$Id: nkf.c,v 1.00 1999/12/07 00:00:00 heller Exp $
___________________________________________________________________________*/
#include "precomp.h"
#include <windows.h>
#include <sys/stat.h>

#define NKF32DLL "nkf32.dll"

typedef void (CALLBACK* __GetNkfVersion)(LPSTR);
typedef int  (CALLBACK* __SetNkfOption )(LPCSTR);
typedef void (CALLBACK* __NkfConver    )(LPSTR, LPCSTR);

/* Bote */
BOOL Isnkf32(void)
{
	HINSTANCE hDLL;					// Handle to DLL
	__GetNkfVersion GetNkfVersion;	// Function pointer
	__SetNkfOption  SetNkfOption;	// Function pointer
	__NkfConver     NkfConvert;		// Function pointer

	hDLL = LoadLibrary(NKF32DLL);
	if (hDLL == NULL) {
		return FALSE;
	} else {
		GetNkfVersion = (__GetNkfVersion)GetProcAddress(hDLL, "GetNkfVersion");
		SetNkfOption  = (__SetNkfOption )GetProcAddress(hDLL, "SetNkfOption" );
		NkfConvert    = (__NkfConver    )GetProcAddress(hDLL, "NkfConvert"   );

		if (!GetNkfVersion || !SetNkfOption || !NkfConvert) {
			// handle the error
			FreeLibrary(hDLL);
			return FALSE;
		} else {
			FreeLibrary(hDLL);
			return TRUE;
		}
	}
}

/* Bote */
int ConvertNKF32(LPCSTR options, LPCSTR pInput, LPSTR pOutput)
{
	int   ReturnValue = 0;
	LPSTR NkfVersion  = NULL;
	HINSTANCE hDLL;					// Handle to DLL
	__GetNkfVersion GetNkfVersion;	// Function pointer
	__SetNkfOption  SetNkfOption;	// Function pointer
	__NkfConver     NkfConvert;		// Function pointer

	hDLL = LoadLibrary(NKF32DLL);
	if (hDLL == NULL) {
		return -1;
	} else {
		GetNkfVersion = (__GetNkfVersion)GetProcAddress(hDLL, "GetNkfVersion");
		SetNkfOption  = (__SetNkfOption )GetProcAddress(hDLL, "SetNkfOption" );
		NkfConvert    = (__NkfConver    )GetProcAddress(hDLL, "NkfConvert"   );

		if (!GetNkfVersion || !SetNkfOption || !NkfConvert) {
			// handle the error
			FreeLibrary(hDLL);       
			return -1;
		} else {
			NkfVersion = (LPSTR)malloc(1024);

			if (NkfVersion) {
				memset(NkfVersion, 0x00, 1024);
				GetNkfVersion(NkfVersion);
				if (strcmp(NkfVersion, "") != 0) {
					ReturnValue = SetNkfOption(options);
					NkfConvert(pOutput, pInput);
				} else {
					ReturnValue = -1;
				}
				memset(NkfVersion, 0x00, 1024);
				free(NkfVersion);
			}
			FreeLibrary(hDLL);
			return ReturnValue;
		}
	}
}

/* Bote */
void ClipboardKANJI(HWND hwnd, LPCSTR options)
{
	void *pClipboardData;
	UINT ClipboardFormat = 0;
	DWORD dwDataSize = 0;
	LPSTR pOutput    = NULL;
	DWORD dwOutSize  = 0;
	DWORD dwOutDataSize  = 0;
 	
	pClipboardData = RetrieveClipboardData(hwnd, &ClipboardFormat, &dwDataSize);

	if (pClipboardData) {
		if (ClipboardFormat == CF_TEXT) {
			dwOutSize = (dwDataSize+1) * 4;
			pOutput   = (LPSTR)malloc(dwOutSize);
			if (pOutput) {
				memset(pOutput, 0x00, dwOutSize);
				if (ConvertNKF32(options, pClipboardData, pOutput) == 0) {
					dwOutDataSize = strlen(pOutput) + 1;
					StoreClipboardData(hwnd, pOutput, dwOutDataSize);
				}
				memset(pOutput, 0x00, dwOutSize);
				free(pOutput);
			}
		}
		memset(pClipboardData, 0x00, dwDataSize);
		free(pClipboardData);
	}
}

/* Bote */
BOOL ClipboardSJIS(HWND hwnd)
{
	ClipboardKANJI(hwnd, "-sm");
	return TRUE;
}

/* Bote */
BOOL ClipboardJIS(HWND hwnd)
{
	ClipboardKANJI(hwnd, "-jm");
	return TRUE;
}

BOOL BufferKANJI(char *pInput, DWORD dwInSize, char **ppOutput, DWORD *pOutSize, LPCSTR options)
{
	DWORD dwMemSize = ((dwInSize) + 1) * 4;

	if (!pInput) return TRUE;

	*ppOutput = (LPSTR)malloc(dwMemSize);

	if (!*ppOutput)
		return FALSE;

	memset(*ppOutput, 0x00, dwMemSize);
	if (ConvertNKF32(options, pInput, *ppOutput) != 0) {
		memset(*ppOutput,0x00,dwMemSize);
		free(*ppOutput);
		*ppOutput = NULL;
		return FALSE;
	}

	*pOutSize = strlen(*ppOutput);
	return TRUE;
}

BOOL BufferSJIS(char *pInput, DWORD dwInSize, char **ppOutput, DWORD *pOutSize)
{
	return BufferKANJI(pInput, dwInSize, ppOutput, pOutSize, "-Jsm");
}

BOOL BufferJIS(char *pInput, DWORD dwInSize, char **ppOutput, DWORD *pOutSize)
{
	return BufferKANJI(pInput, dwInSize, ppOutput, pOutSize, "-Sjm");
}

BOOL FileKANJI(LPCSTR inputFile, LPCSTR outputFile, LPCSTR options)
{
	FILE		   *fpIn;
	FILE		   *fpOut;
	struct stat		statBuf;
	DWORD			BufSize;
	char		   *pBuffer;
	DWORD			len;
	int 			eof;
	char		   *pBufferConv   = NULL;
	DWORD			dwBufSizeConv =    0;

	if ((stat(inputFile, &statBuf) == 0) && ((S_IFMT & statBuf.st_mode) == S_IFREG)) {
		BufSize = statBuf.st_size + 1;
	} else {
		return FALSE;
	}
	
	pBuffer = (LPSTR)malloc(BufSize);
	if (!pBuffer) {
		return FALSE;
	}
	memset(pBuffer, 0x00, BufSize);
	
	if ((fpIn = fopen(inputFile, "rb")) == NULL) {
		return FALSE;
	}
	if ( fseek( fpIn, 0L, SEEK_SET ) != 0 ) {
		fclose(fpIn);
		memset(pBuffer, 0x00, BufSize);
		free( pBuffer );
		return FALSE;
	}
	len = fread(pBuffer, 1, BufSize - 1, fpIn);
	eof = feof(fpIn);
	fclose(fpIn);
	if (len != (BufSize - 1) && !eof) {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		return FALSE;
	}
	BufSize = strlen(pBuffer);
	
	if (BufferKANJI(pBuffer, BufSize, &pBufferConv, &dwBufSizeConv, options)) {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		pBuffer = pBufferConv;
		BufSize = dwBufSizeConv;
	} else {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		return FALSE;
	}
	
	if ((fpOut = fopen(outputFile, "wb")) == NULL) {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		return FALSE;
	}
	len = fwrite(pBuffer, 1, BufSize, fpOut);
	fclose(fpOut);
	memset(pBuffer, 0x00, BufSize);
	free(pBuffer);
	if (len != BufSize) {
		remove(outputFile);
		return FALSE;
	}
	
	return TRUE;
}

BOOL FileSJIS(LPCSTR inputFile, LPCSTR outputFile)
{
	return FileKANJI(inputFile, outputFile, "-Jsm");
}

BOOL FileJIS(LPCSTR inputFile, LPCSTR outputFile)
{
	return FileKANJI(inputFile, outputFile, "-Sjm");
}

BOOL Buffer2FileSJIS(char *pInput, DWORD dwInSize, LPCSTR outputFile)
{
	FILE		   *fpOut;
	DWORD			BufSize;
	char		   *pBuffer;
	DWORD			len;

	if (!BufferSJIS(pInput, dwInSize, &pBuffer, &BufSize)) {
		return FALSE;
	}
	
	if ((fpOut = fopen(outputFile, "wb")) == NULL) {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		return FALSE;
	}
	len = fwrite(pBuffer, 1, BufSize, fpOut);
	fclose(fpOut);
	memset(pBuffer, 0x00, BufSize);
	free(pBuffer);
	if (len != BufSize) {
		remove(outputFile);
		return FALSE;
	}
	
	return TRUE;
}

BOOL File2BufferJIS(LPCSTR inputFile, char **ppOutput, DWORD *pOutSize)
{
	FILE		   *fpIn;
	struct stat		statBuf;
	DWORD			BufSize;
	char		   *pBuffer;
	DWORD			len;
	int				eof;
	char		   *pBufferJIS   = NULL;
	DWORD			dwBufSizeJIS =    0;

	if ((stat(inputFile, &statBuf) == 0) && ((S_IFMT & statBuf.st_mode) == S_IFREG)) {
		BufSize = statBuf.st_size + 1;
	} else {
		return FALSE;
	}
	
	pBuffer = (LPSTR)malloc(BufSize);
	if (!pBuffer) {
		return FALSE;
	}
	memset(pBuffer, 0x00, BufSize);
	
	if ((fpIn = fopen(inputFile, "rb")) == NULL) {
		return FALSE;
	}
	if ( fseek( fpIn, 0L, SEEK_SET ) != 0 ) {
		fclose(fpIn);
		memset(pBuffer, 0x00, BufSize);
		free( pBuffer );
		return FALSE;
	}
	len = fread(pBuffer, 1, BufSize - 1, fpIn);
	eof = feof(fpIn);
	fclose(fpIn);
	if (len != (BufSize - 1) && !eof) {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		return FALSE;
	}
	BufSize = strlen(pBuffer);
	
	if (BufferJIS(pBuffer, BufSize, &pBufferJIS, &dwBufSizeJIS)) {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		pBuffer = pBufferJIS;
		BufSize = dwBufSizeJIS;
	} else {
		memset(pBuffer, 0x00, BufSize);
		free(pBuffer);
		return FALSE;
	}
	*pOutSize = BufSize;
	*ppOutput = pBuffer;
	
	return TRUE;
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
